#define CONTEXTMENU_H

/****************************************************************

    Copyright (c) 1996-1997 Fred Pesch

    This program is free software; you can redistribute it 
    and/or modify it under the terms of the GNU General 
    Public License as published by the Free Software 
    Foundation (version 2 of the License).

    This program is distributed in the hope that it will 
    be useful, but WITHOUT ANY WARRANTY; without even the 
    implied warranty of MERCHANTABILITY or FITNESS FOR A 
    PARTICULAR PURPOSE.  See the GNU General Public License 
    for more details.

    For a copy of the GNU General Public License, write to the 
    Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
    MA 02139, USA.

****************************************************************/

#include <Xm/Xm.h>
#include <Xm/XmAll.h>
#include <stdio.h>

#include "interface.h"
#include "config.h"
#include "fileview.h"
#include "dirtree.h"
#include "dialogs.h"
#include "command.h"
#include "select.h"

struct CMenuNode {
  TMenuType type;
  Widget w_node;
  char *command;
};

typedef struct CMenuNode TCMenuNode;

Widget W_CMenuDir=NULL;
TCMenuNode *G_CMenuDir=NULL;
int G_NbCMenuDir=0;

Widget W_CMenuFile=NULL;
TCMenuNode *G_CMenuFile=NULL;
int G_NbCMenuFile=0;

Widget W_CMenuOther=NULL;
TCMenuNode *G_CMenuOther=NULL;
int G_NbCMenuOther=0;

Widget W_CMenuNoFile=NULL;
TCMenuNode *G_CMenuNoFile=NULL;
int G_NbCMenuNoFile=0;

Widget W_CMenuSelection=NULL;
TCMenuNode *G_CMenuSelection=NULL;
int G_NbCMenuSelection=0;

extern void SelectionCB(Widget w_notebook, XtPointer app_data, XtPointer widget_data);

void ContextMenuEvent( Widget w, XtPointer clientdata, XEvent *event, Boolean *flag)
{
  int xmodif,ymodif;
  Window child;
  Position xmax, ymax;
  Position x,y;
  Dimension width, height;
  Widget w_dest;
  int i;
  int i_dest;
  WidgetList selection;
  XmContainerSelectCallbackStruct * infos;
  Position xadd, yadd;
  

  if(event->xbutton.button == Button3) {

    /* is the cursor in the file list ? */

    XTranslateCoordinates(XtDisplay(W_MainWindow), XtWindow(W_MainWindow), XtWindow(W_ListFile), event->xbutton.x, event->xbutton.y, &xmodif, &ymodif, &child);
 
    XtVaGetValues(XtParent(W_ListFile), XmNwidth, &xmax, XmNheight, &ymax, NULL);
    XtVaGetValues(W_ListFile, XmNx, &xadd, XmNy, &yadd, NULL);

    xmodif+=xadd;
    ymodif+=yadd;

    if(xmodif>=0 && xmodif<=xmax && ymodif>=0 && ymodif<=ymax) {
      /* yes */

      xmodif-=xadd;
      ymodif-=yadd;

      /* if there's a selection of more than one file */
      if(SelectFileViewList!=NULL)
	if(SelectFileViewList->next!=NULL) {
	  XmMenuPosition( W_CMenuSelection, (XButtonPressedEvent *) event);
	  XtManageChild( W_CMenuSelection);
	  return;
	}
	  

      w_dest=NULL;  /* no icon widget */
      
      /* if there's no selection */
      if(SelectFileViewList==NULL) {
	if(FileIconList.nb_icons_managed>0) {
	  for(i=0;i<FileIconList.nb_icons_managed;i++) {
	    XtVaGetValues(FileIconList.w_icon[i], XmNx, &x, XmNy, &y, XmNwidth, &width, XmNheight, &height, NULL);
	    if(xmodif>=x && xmodif<=x+width && ymodif>=y && ymodif<=y+height) {
	      i_dest=i;
	      w_dest=FileIconList.w_icon[i];
	    }
	  }
	}

	/* if we didn't choose an icon, use the "no file" menu */
	if(w_dest==NULL) {
	  XmMenuPosition( W_CMenuNoFile, (XButtonPressedEvent *) event);
	  XtManageChild( W_CMenuNoFile);
	  return;
	}
      }

      /* if the selection is NULL, then select the file */

      if(w_dest!=NULL) {
	selection=(WidgetList) XtMalloc(sizeof(Widget));
	selection[0]=w_dest;
	
	XtVaSetValues(W_ListFile, XmNselectedObjectCount, 1, XmNselectedObjects, selection, NULL);

	infos = (XmContainerSelectCallbackStruct *) XtMalloc(sizeof(XmContainerSelectCallbackStruct));
	infos->selected_item_count = 1;
	infos->selected_items = selection;

	SelectionCB(NULL, NULL, infos);
	
	XtFree((char *) selection);
	XtFree((char *) infos);
      }

      /* then use the appropriate menu */

      switch(SelectFileViewList->filenode->file.type) {
      case SLINK_FILE:
      case REGULAR_FILE:
      case EXECUTABLE_FILE:
	XmMenuPosition( W_CMenuFile, (XButtonPressedEvent *) event);
	XtManageChild( W_CMenuFile);
	break;
      case SLINK_DIR:
      case DIRECTORY:
	XmMenuPosition( W_CMenuDir, (XButtonPressedEvent *) event);
	XtManageChild( W_CMenuDir);
	break;
      default:
	XmMenuPosition( W_CMenuOther, (XButtonPressedEvent *) event);
	XtManageChild( W_CMenuOther);
      }

      return;
	
    }

    /* is the cursor in the dir tree ? */

    /*    XTranslateCoordinates(XtDisplay(W_MainWindow), XtWindow(W_MainWindow), XtWindow(W_TreeContainer), event->xbutton.x, event->xbutton.y, &xmodif, &ymodif, &child);
 
    XtVaGetValues(XtParent(W_TreeContainer), XmNwidth, &xmax, XmNheight, &ymax, NULL);
    XtVaGetValues(W_TreeContainer, XmNx, &xadd, XmNy, &yadd, NULL);

    xmodif+=xadd;
    ymodif+=yadd;
    */    
    /*    if(xmodif>=0 && xmodif<=xmax && ymodif>=0 && ymodif<=ymax) {*/
      /* yes */

    /*      xmodif-=xadd;
      ymodif-=yadd;

      */   /* get selected icon */

    /*      w_dest=NULL;*/  /* no icon widget */
    /*      if(DirIconList.nb_icons_managed>0) {
	for(i=0;i<DirIconList.nb_icons_managed;i++) {
	  XtVaGetValues(DirIconList.w_icon[i], XmNx, &x, XmNy, &y, XmNwidth, &width, XmNheight, &height, NULL);
	  if(xmodif>=x && xmodif<=x+width && ymodif>=y && ymodif<=y+height) {
	    i_dest=i;
	    w_dest=DirIconList.w_icon[i];
	  }
	}
      }

      if(w_dest!=NULL) {
	XmMenuPosition( W_CMenuDir, (XButtonPressedEvent *) event);
	XtManageChild( W_CMenuDir);
	return;
      } else {
	XmMenuPosition( W_CMenuNoFile, (XButtonPressedEvent *) event);
	XtManageChild( W_CMenuNoFile);
	return;
      }
      

    }*/

    /* else use the "no file" menu */

    XmMenuPosition( W_CMenuNoFile, (XButtonPressedEvent *) event);
    XtManageChild( W_CMenuNoFile);

  }
}

TCMenuNode *SearchCMenuNode(TCMenuNode *list, int nb_elem, char *name)
{
  TCMenuNode *result;
  char *wname;
  int i; 

  result=NULL;
  for(i=0;i<nb_elem;i++) {
    wname=XtName(list[i].w_node);
    if(strcmp(wname,name)==0) result=&list[i];
  }
  return result;
}
  

void ContextMenuDirCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XmRowColumnCallbackStruct * infos;
  char *wname;
  char *destdir;
  TCMenuNode *usernode;

  infos = (XmRowColumnCallbackStruct *) widget_data;

  wname = XtName(infos->widget);  

  if(strcmp(wname, "ButtonCd")==0) {
    if(SelectFileViewList!=NULL) {
      destdir=XtMalloc(strlen(SelectFileViewList->filenode->file.name)+strlen(ActualDir->dir.pathname)+2);
      strcpy(destdir,ActualDir->dir.pathname);
      strcat(destdir,"/");
      strcat(destdir,SelectFileViewList->filenode->file.name);
      GoToDirRefreshRoutine(destdir);
      XtFree(destdir);
      return;
    }
  }  

  if(strcmp(wname, "ButtonNewCd")==0) {
    if(SelectFileViewList!=NULL) {
      destdir=XtMalloc(strlen(SelectFileViewList->filenode->file.name)+strlen(ActualDir->dir.pathname)+2);
      strcpy(destdir,ActualDir->dir.pathname);
      strcat(destdir,"/");
      strcat(destdir,SelectFileViewList->filenode->file.name);
      GoToDirNewWindow(destdir);
      XtFree(destdir);
      return;
    }
  }


  if(strcmp(wname, "ButtonHome")==0) {
    HomeDirRoutine();
    return;
  }

  if(strcmp(wname, "ButtonGoto")==0) {
    GoToDirRoutine();
    return;
  }
 
  if(strcmp(wname, "ButtonInfos")==0) {
    FileInfoRoutine();
    return;
  }
   
  if(strcmp(wname, "ButtonDelete")==0) {
    FileDeleteRoutine();
    return;
  }

  usernode=SearchCMenuNode(G_CMenuDir, G_NbCMenuDir, wname);
  if(usernode!=NULL && usernode->command!=NULL) 
    UserMenuCommandRoutine(usernode->command);

}

void ContextMenuFileCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XmRowColumnCallbackStruct * infos;
  char *wname;
  TCMenuNode *usernode;

  infos = (XmRowColumnCallbackStruct *) widget_data;

  wname = XtName(infos->widget);  

  if(strcmp(wname, "ButtonView")==0) {
    FileViewCommandRoutine();
    return;
  }

  if(strcmp(wname, "ButtonEdit")==0) {
    FileEditCommandRoutine();
    return;
  }

  if(strcmp(wname, "ButtonInfos")==0) {
    FileInfoRoutine();
    return;
  }

  if(strcmp(wname, "ButtonRename")==0) {
    FileRenameRoutine();
    return;
  }

  if(strcmp(wname, "ButtonDelete")==0) {
    FileDeleteRoutine();
    return;
  }

  if(strcmp(wname, "ButtonExec")==0) {
    FileExecRoutine();
    return;
  }
   
  usernode=SearchCMenuNode(G_CMenuFile, G_NbCMenuFile, wname);
  if(usernode!=NULL && usernode->command!=NULL) 
    UserMenuCommandRoutine(usernode->command);

}

void ContextMenuOtherCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XmRowColumnCallbackStruct * infos;
  char *wname;
  TCMenuNode *usernode;

  infos = (XmRowColumnCallbackStruct *) widget_data;

  wname = XtName(infos->widget);  

  if(strcmp(wname, "ButtonInfos")==0) {
    FileInfoRoutine();
    return;
  }

  usernode=SearchCMenuNode(G_CMenuOther, G_NbCMenuOther, wname);
  if(usernode!=NULL && usernode->command!=NULL) 
    UserMenuCommandRoutine(usernode->command);

}

void ContextMenuNoFileCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XmRowColumnCallbackStruct * infos;
  char *wname;
  TCMenuNode *usernode;

  infos = (XmRowColumnCallbackStruct *) widget_data;

  wname = XtName(infos->widget);  

  if(strcmp(wname, "ButtonCreate")==0) {
    FileCreateRoutine();
    return;
  }

  if(strcmp(wname, "ButtonSelectAll")==0) {
    SelectAllCommand();
    return;
  }

  if(strcmp(wname, "ButtonFilterSelect")==0) {
    SelectFilterRoutine();
    return;
  }

  usernode=SearchCMenuNode(G_CMenuNoFile, G_NbCMenuNoFile, wname);
  if(usernode!=NULL && usernode->command!=NULL) 
    UserMenuCommandRoutine(usernode->command);

}

void ContextMenuSelectionCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XmRowColumnCallbackStruct * infos;
  char *wname;
  TCMenuNode *usernode;

  infos = (XmRowColumnCallbackStruct *) widget_data;

  wname = XtName(infos->widget);  

  if(strcmp(wname, "ButtonCopy")==0) {
    FileCopyRoutine();
    return;
  }

  if(strcmp(wname, "ButtonMove")==0) {
    FileMoveRoutine();
    return;
  }

  if(strcmp(wname, "ButtonDelete")==0) {
    FileDeleteRoutine();
    return;
  }

  if(strcmp(wname, "ButtonTar")==0) {
    ArchiveTarRoutine();
    return;
  }

  usernode=SearchCMenuNode(G_CMenuSelection, G_NbCMenuSelection, wname);
  if(usernode!=NULL && usernode->command!=NULL) 
    UserMenuCommandRoutine(usernode->command);

}


void CreateContextMenus(void)
{
  Widget w_temp;
  int i,j;
  XmString xstrtemp;
  int found;
  char btnname[32];

  /*---------------- context menu : directory ------------------*/

  W_CMenuDir = XmCreatePopupMenu(W_MainWindow, "DirContextMenu", NULL, 0);
  
  w_temp=XtVaCreateManagedWidget("Label", xmLabelWidgetClass, W_CMenuDir, NULL);

  XtVaCreateManagedWidget("Separator1", xmSeparatorWidgetClass, W_CMenuDir, NULL);

  XtVaCreateManagedWidget("ButtonCd", xmPushButtonWidgetClass, W_CMenuDir, NULL);

  XtVaCreateManagedWidget("ButtonNewCd", xmPushButtonWidgetClass, W_CMenuDir, NULL);

  XtVaCreateManagedWidget("Separator2", xmSeparatorWidgetClass, W_CMenuDir, NULL);

  XtVaCreateManagedWidget("ButtonHome", xmPushButtonWidgetClass, W_CMenuDir, NULL);

  XtVaCreateManagedWidget("ButtonGoto", xmPushButtonWidgetClass, W_CMenuDir, NULL);

  XtVaCreateManagedWidget("Separator3", xmSeparatorWidgetClass, W_CMenuDir, NULL);
  
  XtVaCreateManagedWidget("ButtonInfos", xmPushButtonWidgetClass, W_CMenuDir, NULL);

  XtVaCreateManagedWidget("Separator4", xmSeparatorWidgetClass, W_CMenuDir, NULL);
  
  XtVaCreateManagedWidget("ButtonDelete", xmPushButtonWidgetClass, W_CMenuDir, NULL);

  /* parse the user menu commands */

  found=0;
  i=0;
  while(i<G_NbCMenus && !found) {
    if(strcmp(G_CMenuList[i].ident,"DirMenu")==0)
      found=1;
    else i++;
  }

  if(found) {

    xstrtemp=XmStringCreateSimple(G_CMenuList[i].name);

    XtVaSetValues(w_temp, XmNlabelString, xstrtemp, NULL);

    XmStringFree(xstrtemp);

    G_NbCMenuDir = G_CMenuList[i].nb_entries;
    
    if(G_CMenuList[i].nb_entries!=0) {
	
      XtVaCreateManagedWidget("Separator5", xmSeparatorWidgetClass, W_CMenuDir, NULL);

      G_CMenuDir = (TCMenuNode *) XtMalloc(sizeof(TCMenuNode) * G_CMenuList[i].nb_entries);
      for(j=0;j<G_CMenuList[i].nb_entries; j++) {
	G_CMenuDir[j].type=G_CMenuList[i].entry[j].type;
	
	xstrtemp=XmStringCreateSimple(G_CMenuList[i].entry[j].name);
	sprintf(btnname,"ButtonUser%d",j);
	G_CMenuDir[j].w_node = XtVaCreateManagedWidget(btnname, xmPushButtonWidgetClass, W_CMenuDir, XmNlabelString, xstrtemp, NULL);
	XmStringFree(xstrtemp);

	G_CMenuDir[j].command = XtMalloc(sizeof(char)* (strlen(G_CMenuList[i].entry[j].command)+1));
	
	strcpy(G_CMenuDir[j].command, G_CMenuList[i].entry[j].command);
      }
    }
  }

  /* add a menu callback */

  XtAddCallback( W_CMenuDir, XmNentryCallback, ContextMenuDirCB, NULL);

  /*------------------- context menu : file ----------------*/

  W_CMenuFile = XmCreatePopupMenu(W_MainWindow, "FileContextMenu", NULL, 0);
  
  w_temp=XtVaCreateManagedWidget("Label", xmLabelWidgetClass, W_CMenuFile, NULL);

  XtVaCreateManagedWidget("Separator1", xmSeparatorWidgetClass, W_CMenuFile, NULL);

  XtVaCreateManagedWidget("ButtonView", xmPushButtonWidgetClass, W_CMenuFile, NULL);

  XtVaCreateManagedWidget("ButtonEdit", xmPushButtonWidgetClass, W_CMenuFile, NULL);

  XtVaCreateManagedWidget("ButtonInfos", xmPushButtonWidgetClass, W_CMenuFile, NULL);

  XtVaCreateManagedWidget("Separator2", xmSeparatorWidgetClass, W_CMenuFile, NULL);

  XtVaCreateManagedWidget("ButtonRename", xmPushButtonWidgetClass, W_CMenuFile, NULL);

  XtVaCreateManagedWidget("ButtonDelete", xmPushButtonWidgetClass, W_CMenuFile, NULL);

  XtVaCreateManagedWidget("Separator3", xmSeparatorWidgetClass, W_CMenuFile, NULL);
  
  XtVaCreateManagedWidget("ButtonExec", xmPushButtonWidgetClass, W_CMenuFile, NULL);

  /* parse the user menu commands */

  found=0;
  i=0;
  while(i<G_NbCMenus && !found) {
    if(strcmp(G_CMenuList[i].ident,"FileMenu")==0)
      found=1;
    else i++;
  }

  if(found) {

    xstrtemp=XmStringCreateSimple(G_CMenuList[i].name);

    XtVaSetValues(w_temp, XmNlabelString, xstrtemp, NULL);

    XmStringFree(xstrtemp);

    G_NbCMenuFile = G_CMenuList[i].nb_entries;
    
    if(G_CMenuList[i].nb_entries!=0) {
	
      XtVaCreateManagedWidget("Separator4", xmSeparatorWidgetClass, W_CMenuFile, NULL);

      G_CMenuFile = (TCMenuNode *) XtMalloc(sizeof(TCMenuNode) * G_CMenuList[i].nb_entries);
      for(j=0;j<G_CMenuList[i].nb_entries; j++) {
	G_CMenuFile[j].type=G_CMenuList[i].entry[j].type;
	
	xstrtemp=XmStringCreateSimple(G_CMenuList[i].entry[j].name);
	sprintf(btnname,"ButtonUser%d",j);
	G_CMenuFile[j].w_node = XtVaCreateManagedWidget(btnname, xmPushButtonWidgetClass, W_CMenuFile, XmNlabelString, xstrtemp, NULL);
	XmStringFree(xstrtemp);

	G_CMenuFile[j].command = XtMalloc(sizeof(char)* (strlen(G_CMenuList[i].entry[j].command)+1));
	
	strcpy(G_CMenuFile[j].command, G_CMenuList[i].entry[j].command);
      }
    }
  }

  /* add a menu callback */

  XtAddCallback( W_CMenuFile, XmNentryCallback, ContextMenuFileCB, NULL);

  /*------------------- context menu : other ----------------*/

  W_CMenuOther = XmCreatePopupMenu(W_MainWindow, "OtherContextMenu", NULL, 0);
  
  w_temp=XtVaCreateManagedWidget("Label", xmLabelWidgetClass, W_CMenuOther, NULL);

  XtVaCreateManagedWidget("Separator1", xmSeparatorWidgetClass, W_CMenuOther, NULL);

  XtVaCreateManagedWidget("ButtonInfos", xmPushButtonWidgetClass, W_CMenuOther, NULL);

  /* parse the user menu commands */

  found=0;
  i=0;
  while(i<G_NbCMenus && !found) {
    if(strcmp(G_CMenuList[i].ident,"OtherMenu")==0)
      found=1;
    else i++;
  }

  if(found) {

    xstrtemp=XmStringCreateSimple(G_CMenuList[i].name);

    XtVaSetValues(w_temp, XmNlabelString, xstrtemp, NULL);

    XmStringFree(xstrtemp);

    G_NbCMenuOther = G_CMenuList[i].nb_entries;
    
    if(G_CMenuList[i].nb_entries!=0) {
	
      XtVaCreateManagedWidget("Separator2", xmSeparatorWidgetClass, W_CMenuOther, NULL);

      G_CMenuOther = (TCMenuNode *) XtMalloc(sizeof(TCMenuNode) * G_CMenuList[i].nb_entries);
      for(j=0;j<G_CMenuList[i].nb_entries; j++) {
	G_CMenuOther[j].type=G_CMenuList[i].entry[j].type;
	
	xstrtemp=XmStringCreateSimple(G_CMenuList[i].entry[j].name);
	sprintf(btnname,"ButtonUser%d",j);
	G_CMenuOther[j].w_node = XtVaCreateManagedWidget(btnname, xmPushButtonWidgetClass, W_CMenuOther, XmNlabelString, xstrtemp, NULL);
	XmStringFree(xstrtemp);

	G_CMenuOther[j].command = XtMalloc(sizeof(char)* (strlen(G_CMenuList[i].entry[j].command)+1));
	
	strcpy(G_CMenuOther[j].command, G_CMenuList[i].entry[j].command);
      }
    }
  }

  /* add a menu callback */

  XtAddCallback( W_CMenuOther, XmNentryCallback, ContextMenuOtherCB, NULL);


  /*------------------- context menu : no file ----------------*/

  W_CMenuNoFile = XmCreatePopupMenu(W_MainWindow, "NoFileContextMenu", NULL, 0);
  
  w_temp=XtVaCreateManagedWidget("Label", xmLabelWidgetClass, W_CMenuNoFile, NULL);

  XtVaCreateManagedWidget("Separator1", xmSeparatorWidgetClass, W_CMenuNoFile, NULL);

  XtVaCreateManagedWidget("ButtonCreate", xmPushButtonWidgetClass, W_CMenuNoFile, NULL);

  XtVaCreateManagedWidget("Separator2", xmSeparatorWidgetClass, W_CMenuNoFile, NULL);

  XtVaCreateManagedWidget("ButtonSelectAll", xmPushButtonWidgetClass, W_CMenuNoFile, NULL);

  XtVaCreateManagedWidget("ButtonFilterSelect", xmPushButtonWidgetClass, W_CMenuNoFile, NULL);

  /* parse the user menu commands */

  found=0;
  i=0;
  while(i<G_NbCMenus && !found) {
    if(strcmp(G_CMenuList[i].ident,"NoFileMenu")==0)
      found=1;
    else i++;
  }

  if(found) {

    xstrtemp=XmStringCreateSimple(G_CMenuList[i].name);

    XtVaSetValues(w_temp, XmNlabelString, xstrtemp, NULL);

    XmStringFree(xstrtemp);

    G_NbCMenuNoFile = G_CMenuList[i].nb_entries;
    
    if(G_CMenuList[i].nb_entries!=0) {
	
      XtVaCreateManagedWidget("Separator3", xmSeparatorWidgetClass, W_CMenuNoFile, NULL);

      G_CMenuNoFile = (TCMenuNode *) XtMalloc(sizeof(TCMenuNode) * G_CMenuList[i].nb_entries);
      for(j=0;j<G_CMenuList[i].nb_entries; j++) {
	G_CMenuNoFile[j].type=G_CMenuList[i].entry[j].type;
	
	xstrtemp=XmStringCreateSimple(G_CMenuList[i].entry[j].name);
	sprintf(btnname,"ButtonUser%d",j);
	G_CMenuNoFile[j].w_node = XtVaCreateManagedWidget(btnname, xmPushButtonWidgetClass, W_CMenuNoFile, XmNlabelString, xstrtemp, NULL);
	XmStringFree(xstrtemp);

	G_CMenuNoFile[j].command = XtMalloc(sizeof(char)* (strlen(G_CMenuList[i].entry[j].command)+1));
	
	strcpy(G_CMenuNoFile[j].command, G_CMenuList[i].entry[j].command);
      }
    }
  }

  /* add a menu callback */

  XtAddCallback( W_CMenuNoFile, XmNentryCallback, ContextMenuNoFileCB, NULL);


  /*------------------- context menu : selection ----------------*/

  W_CMenuSelection = XmCreatePopupMenu(W_MainWindow, "SelectionContextMenu", NULL, 0);
  
  w_temp=XtVaCreateManagedWidget("Label", xmLabelWidgetClass, W_CMenuSelection, NULL);

  XtVaCreateManagedWidget("Separator1", xmSeparatorWidgetClass, W_CMenuSelection, NULL);

  XtVaCreateManagedWidget("ButtonCopy", xmPushButtonWidgetClass, W_CMenuSelection, NULL);

  XtVaCreateManagedWidget("ButtonMove", xmPushButtonWidgetClass, W_CMenuSelection, NULL);

  XtVaCreateManagedWidget("ButtonDelete", xmPushButtonWidgetClass, W_CMenuSelection, NULL);

  XtVaCreateManagedWidget("Separator2", xmSeparatorWidgetClass, W_CMenuSelection, NULL);

  XtVaCreateManagedWidget("ButtonTar", xmPushButtonWidgetClass, W_CMenuSelection, NULL);

  /* parse the user menu commands */

  found=0;
  i=0;
  while(i<G_NbCMenus && !found) {
    if(strcmp(G_CMenuList[i].ident,"SelectionMenu")==0)
      found=1;
    else i++;
  }

  if(found) {

    xstrtemp=XmStringCreateSimple(G_CMenuList[i].name);

    XtVaSetValues(w_temp, XmNlabelString, xstrtemp, NULL);

    XmStringFree(xstrtemp);

    G_NbCMenuSelection = G_CMenuList[i].nb_entries;
    
    if(G_CMenuList[i].nb_entries!=0) {
	
      XtVaCreateManagedWidget("Separator3", xmSeparatorWidgetClass, W_CMenuSelection, NULL);

      G_CMenuSelection = (TCMenuNode *) XtMalloc(sizeof(TCMenuNode) * G_CMenuList[i].nb_entries);
      for(j=0;j<G_CMenuList[i].nb_entries; j++) {
	G_CMenuSelection[j].type=G_CMenuList[i].entry[j].type;
	
	xstrtemp=XmStringCreateSimple(G_CMenuList[i].entry[j].name);
	sprintf(btnname,"ButtonUser%d",j);
	G_CMenuSelection[j].w_node = XtVaCreateManagedWidget(btnname, xmPushButtonWidgetClass, W_CMenuSelection, XmNlabelString, xstrtemp, NULL);
	XmStringFree(xstrtemp);

	G_CMenuSelection[j].command = XtMalloc(sizeof(char)* (strlen(G_CMenuList[i].entry[j].command)+1));
	
	strcpy(G_CMenuSelection[j].command, G_CMenuList[i].entry[j].command);
      }
    }
  }

  /* add a menu callback */

  XtAddCallback( W_CMenuSelection, XmNentryCallback, ContextMenuSelectionCB, NULL);


  /*------------------- the menu event handlers ------------------*/

  XtAddEventHandler( W_MainWindow, ButtonPressMask, FALSE, ContextMenuEvent, NULL);

}

void DestroyContextMenus(void)
{
  int i;
  Widget w_temp;

  /* context menu : dir */

  w_temp = XtNameToWidget(W_CMenuDir, "Label");
  XtDestroyWidget(w_temp);

  w_temp = XtNameToWidget(W_CMenuDir, "Separator1");
  XtDestroyWidget(w_temp);

  w_temp = XtNameToWidget(W_CMenuDir, "ButtonCd");
  XtDestroyWidget(w_temp);

  w_temp = XtNameToWidget(W_CMenuDir, "ButtonNewCd");
  XtDestroyWidget(w_temp);

  w_temp = XtNameToWidget(W_CMenuDir, "Separator2");
  XtDestroyWidget(w_temp);

  w_temp = XtNameToWidget(W_CMenuDir, "ButtonHome");
  XtDestroyWidget(w_temp);

  w_temp = XtNameToWidget(W_CMenuDir, "ButtonGoto");
  XtDestroyWidget(w_temp);

  w_temp = XtNameToWidget(W_CMenuDir, "Separator3");
  XtDestroyWidget(w_temp);

  w_temp = XtNameToWidget(W_CMenuDir, "ButtonInfos");
  XtDestroyWidget(w_temp);

  w_temp = XtNameToWidget(W_CMenuDir, "Separator5");
  XtDestroyWidget(w_temp);

  w_temp = XtNameToWidget(W_CMenuDir, "ButtonDelete");
  XtDestroyWidget(w_temp);

  if(G_NbCMenuDir>0) {

    w_temp = XtNameToWidget(W_CMenuDir, "Separator5");
    XtDestroyWidget(w_temp);
    
    for(i=0;i<G_NbCMenuDir;i++) {
      XtDestroyWidget(G_CMenuDir[i].w_node);
      XtFree(G_CMenuDir[i].command);
    }

    XtDestroyWidget(W_CMenuDir);

    XtFree((char *) G_CMenuDir);
    G_NbCMenuDir=0;
  }

  /* context menu : file */

  w_temp = XtNameToWidget(W_CMenuFile, "Label");
  XtDestroyWidget(w_temp);

  w_temp = XtNameToWidget(W_CMenuFile, "Separator1");
  XtDestroyWidget(w_temp);

  w_temp = XtNameToWidget(W_CMenuFile, "ButtonView");
  XtDestroyWidget(w_temp);

  w_temp = XtNameToWidget(W_CMenuFile, "ButtonEdit");
  XtDestroyWidget(w_temp);

  w_temp = XtNameToWidget(W_CMenuFile, "ButtonInfos");
  XtDestroyWidget(w_temp);

  w_temp = XtNameToWidget(W_CMenuFile, "Separator2");
  XtDestroyWidget(w_temp);

  w_temp = XtNameToWidget(W_CMenuFile, "ButtonRename");
  XtDestroyWidget(w_temp);

  w_temp = XtNameToWidget(W_CMenuFile, "ButtonDelete");
  XtDestroyWidget(w_temp);

  w_temp = XtNameToWidget(W_CMenuFile, "Separator3");
  XtDestroyWidget(w_temp);

  w_temp = XtNameToWidget(W_CMenuFile, "ButtonExec");
  XtDestroyWidget(w_temp);

  if(G_NbCMenuFile>0) {

    w_temp = XtNameToWidget(W_CMenuFile, "Separator4");
    XtDestroyWidget(w_temp);
    
    for(i=0;i<G_NbCMenuFile;i++) {
      XtDestroyWidget(G_CMenuFile[i].w_node);
      XtFree(G_CMenuFile[i].command);
    }

    XtDestroyWidget(W_CMenuFile);

    XtFree((char *) G_CMenuFile);
    G_NbCMenuFile=0;
  }

  /* context menu : other */

  w_temp = XtNameToWidget(W_CMenuOther, "Label");
  XtDestroyWidget(w_temp);

  w_temp = XtNameToWidget(W_CMenuOther, "Separator1");
  XtDestroyWidget(w_temp);

  w_temp = XtNameToWidget(W_CMenuOther, "ButtonInfos");
  XtDestroyWidget(w_temp);

  if(G_NbCMenuOther>0) {

    w_temp = XtNameToWidget(W_CMenuOther, "Separator2");
    XtDestroyWidget(w_temp);
    
    for(i=0;i<G_NbCMenuOther;i++) {
      XtDestroyWidget(G_CMenuOther[i].w_node);
      XtFree(G_CMenuOther[i].command);
    }

    XtDestroyWidget(W_CMenuOther);

    XtFree((char *) G_CMenuOther);
    G_NbCMenuOther=0;
  }

  /* context menu : no file */

  w_temp = XtNameToWidget(W_CMenuNoFile, "Label");
  XtDestroyWidget(w_temp);

  w_temp = XtNameToWidget(W_CMenuNoFile, "Separator1");
  XtDestroyWidget(w_temp);

  w_temp = XtNameToWidget(W_CMenuNoFile, "ButtonCreate");
  XtDestroyWidget(w_temp);

  w_temp = XtNameToWidget(W_CMenuNoFile, "Separator2");
  XtDestroyWidget(w_temp);

  w_temp = XtNameToWidget(W_CMenuNoFile, "ButtonSelect");
  XtDestroyWidget(w_temp);

  w_temp = XtNameToWidget(W_CMenuNoFile, "ButtonUnSelect");
  XtDestroyWidget(w_temp);

  if(G_NbCMenuNoFile>0) {

    w_temp = XtNameToWidget(W_CMenuNoFile, "Separator3");
    XtDestroyWidget(w_temp);
    
    for(i=0;i<G_NbCMenuNoFile;i++) {
      XtDestroyWidget(G_CMenuNoFile[i].w_node);
      XtFree(G_CMenuNoFile[i].command);
    }

    XtDestroyWidget(W_CMenuNoFile);

    XtFree((char *) G_CMenuNoFile);
    G_NbCMenuNoFile=0;
  }

  /* context menu : selection */

  w_temp = XtNameToWidget(W_CMenuSelection, "Label");
  XtDestroyWidget(w_temp);

  w_temp = XtNameToWidget(W_CMenuSelection, "Separator1");
  XtDestroyWidget(w_temp);

  w_temp = XtNameToWidget(W_CMenuSelection, "ButtonCopy");
  XtDestroyWidget(w_temp);

  w_temp = XtNameToWidget(W_CMenuSelection, "ButtonMove");
  XtDestroyWidget(w_temp);

  w_temp = XtNameToWidget(W_CMenuSelection, "ButtonDelete");
  XtDestroyWidget(w_temp);

  w_temp = XtNameToWidget(W_CMenuSelection, "Separator2");
  XtDestroyWidget(w_temp);

  w_temp = XtNameToWidget(W_CMenuSelection, "ButtonTar");
  XtDestroyWidget(w_temp);

  if(G_NbCMenuSelection>0) {

    w_temp = XtNameToWidget(W_CMenuSelection, "Separator3");
    XtDestroyWidget(w_temp);
    
    for(i=0;i<G_NbCMenuSelection;i++) {
      XtDestroyWidget(G_CMenuSelection[i].w_node);
      XtFree(G_CMenuSelection[i].command);
    }

    XtDestroyWidget(W_CMenuSelection);

    XtFree((char *) G_CMenuSelection);
    G_NbCMenuSelection=0;
  }

  /* remove the event handler */

  XtRemoveEventHandler( W_MainWindow, ButtonPressMask, FALSE, ContextMenuEvent, NULL);


}






