#define SHELL_H

/****************************************************************

    Copyright (c) 1996-1997 Fred Pesch

    This program is free software; you can redistribute it 
    and/or modify it under the terms of the GNU General 
    Public License as published by the Free Software 
    Foundation (version 2 of the License).

    This program is distributed in the hope that it will 
    be useful, but WITHOUT ANY WARRANTY; without even the 
    implied warranty of MERCHANTABILITY or FITNESS FOR A 
    PARTICULAR PURPOSE.  See the GNU General Public License 
    for more details.

    For a copy of the GNU General Public License, write to the 
    Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
    MA 02139, USA.

****************************************************************/

/* includes */

#include <Xm/Xm.h>
#include <Xm/MainW.h>
#include <Xm/PushB.h>
#include <Xm/Text.h>
#include <Xm/Form.h>
#include <Xm/ScrolledW.h>
#include <Xm/Separator.h>
#include <Xm/List.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/PanedW.h>

#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/wait.h>
#include <sys/types.h>

#include "dirtree.h"
#include "interface.h"
#include "error_handler.h"

/* global variables */

Widget W_ShellDialog=NULL;
Widget W_ShellOutputText=NULL;
Widget W_ShellHistoryList=NULL;
Widget W_ShellCommand=NULL;

XtInputId ShellInputId;

int ShellChildExist=0;

struct sigaction ShellChildAction;
struct sigaction ShellOldAction;

pid_t ShellChildPID;

/* temp */
Widget W_App;
Widget W_MainWindow;
XtAppContext AppContext;

/* Misc functions */

void GetShellResult( XtPointer clientData, int *fid, XtInputId *id)
{
  char buf[BUFSIZ+1];
  int nbbytes;

   nbbytes=read(*fid, buf, BUFSIZ);

   if(nbbytes>0) {
     buf[nbbytes]=(char)0;
     XmTextInsert(W_ShellOutputText, XmTextGetLastPosition(W_ShellOutputText), buf);
   }
}

void ShellChildHandler(int sig)
{
  int status;
  int result;
  Widget w_temp;

  if(!ShellChildExist) return;

  waitpid(ShellChildPID, &status, 0);

  result=sigaction(SIGCHLD, &ShellOldAction, NULL);
  /*  if(result==-1) FatalError("shell.c", "ShellChildHandler","Unable to install the default child signal handler");*/

  XtRemoveInput(ShellInputId);
  ShellChildExist=0;

  w_temp = XtNameToWidget(W_ShellDialog, "*ShellStopButton");
  XtVaSetValues(w_temp, XmNsensitive, False, NULL);

}

void ShellExecCommand(char *cmdstr)
{
  pid_t child;
  int result;
  char *arglist[256];
  int countarg;
  int pipe_to_parent[2];
  int resultex;
  Widget w_temp;

  ShellChildExist=1;
  XmTextSetString(W_ShellOutputText, "");
  XmTextSetInsertionPosition(W_ShellOutputText, 0);
  
  ShellInputId=XtAppAddInput(AppContext, fileno(stdin), (XtPointer) XtInputReadMask, GetShellResult, NULL);

  w_temp = XtNameToWidget(W_ShellDialog, "*ShellStopButton");
  XtVaSetValues(w_temp, XmNsensitive, True, NULL);

  pipe(pipe_to_parent);

  /* first install the child handler */
  ShellChildAction.sa_handler=ShellChildHandler;
  result=sigaction(SIGCHLD, &ShellChildAction, &ShellOldAction);
  /*  if(result==-1) FatalError("shell.c", "ShellExecCommand","Unable to install signal handler");*/
  child=fork();
  /*  if(child==-1) FatalError("shell.c", "ShellExecCommand","unable to fork()");*/
  if(child==0) {
    countarg=0;
    arglist[countarg]=strtok(cmdstr," ");
    while(arglist[countarg]!=NULL) {
      countarg++;
      arglist[countarg]=strtok(NULL, " ");
    }

    close(pipe_to_parent[0]);
    result=dup2(pipe_to_parent[1],fileno(stdout));
    result=dup2(pipe_to_parent[1],fileno(stderr));
    resultex=execvp(arglist[0], arglist);            
    if(resultex==-1) {
      fprintf(stderr,"cannot execute %s",cmdstr);
      sleep(3);
      exit(0);
    }
  }


  ShellChildPID=child;

  result=dup2(pipe_to_parent[0],fileno(stdin));
  close(pipe_to_parent[1]);
  
}


/* Callbacks */

void ShellExecButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  char *command;
  XmString xmcommand;
  int pos;

  if(ShellChildExist) {
    if(ConfirmDialog("Previous command still running", "Do you want to abort the previous command ?")) {
      if(ShellChildExist) {
	kill(ShellChildPID, SIGKILL);
	sleep(3);
      }
    
    } else return;
  } 
  
  command=XmTextGetString(W_ShellCommand);

  if(command!=NULL && strcmp(command,"")!=0) {
    xmcommand=XmStringCreateSimple(command);

    pos = XmListItemPos(W_ShellHistoryList, xmcommand);

    if(pos!=0) XmListDeletePos(W_ShellHistoryList, pos);

    XmListAddItem(W_ShellHistoryList, xmcommand, 1);
    XmStringFree(xmcommand);
    XtVaSetValues(W_ShellHistoryList, XmNvisibleItemCount, 8, NULL);
    ShellExecCommand(command);
  }

  if(command!=NULL)
    XtFree(command);
}

void ShellStopButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  if(ShellChildExist)
    kill(ShellChildPID, SIGKILL);

  XtVaSetValues(w_button, XmNsensitive, False, NULL);

}

void ShellInsSelButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  Widget w_temp;
  TViewNode *actual;

  if(SelectFileViewList==NULL) return;

  w_temp=XtNameToWidget(W_ShellDialog, "*ShellCommand");
  actual=SelectFileViewList;
  while(actual!=NULL) {
    XmTextInsert(w_temp, XmTextGetLastPosition(w_temp), " ");
    XmTextInsert(w_temp, XmTextGetLastPosition(w_temp), actual->filenode->file.name);
    actual=actual->next;
  }

}

void ShellCloseButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  if(ShellChildExist)
    kill(ShellChildPID, SIGKILL);

  XtUnmanageChild(W_ShellDialog);
}


void ShellSelectHistoryCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XmListCallbackStruct *infos;
  char *command;

  if(ShellChildExist) return;

  infos = (XmListCallbackStruct *) widget_data;

  if(infos->reason==XmCR_DEFAULT_ACTION) {
    if(!XmStringGetLtoR(infos->item, XmFONTLIST_DEFAULT_TAG, &command)) return;
    XmTextSetString(W_ShellCommand, command);
    if(command!=NULL) XtFree(command);
  }
}


/* Dialog creation */

void ShellInitDialog(void)
{
  Widget w_frame1, w_frame2, w_frame3, w_scroll1, w_separator, w_button;
  Widget w_label, w_pane;
  Arg args[4];
  int nbargs;


  W_ShellDialog=XmCreateFormDialog(W_MainWindow,"ShellDialog", NULL,0);

  XtVaSetValues(W_ShellDialog, 
		XmNdialogStyle, XmDIALOG_MODELESS,
		XmNautoUnmanage, False,
		NULL);

  w_button = XtVaCreateManagedWidget("ShellHelpButton", xmPushButtonWidgetClass, W_ShellDialog,
				     XmNrightAttachment, XmATTACH_FORM,
				     XmNbottomAttachment, XmATTACH_FORM,
				     NULL);

  w_button = XtVaCreateManagedWidget("ShellCloseButton", xmPushButtonWidgetClass, W_ShellDialog,
				     XmNrightAttachment, XmATTACH_WIDGET,
				     XmNrightWidget, w_button,
				     XmNbottomAttachment, XmATTACH_FORM,
				     NULL);

  XtAddCallback(w_button, XmNactivateCallback, ShellCloseButtonCB, NULL);

  w_button = XtVaCreateManagedWidget("ShellStopButton", xmPushButtonWidgetClass, W_ShellDialog,
				     XmNrightAttachment, XmATTACH_WIDGET,
				     XmNrightWidget, w_button,
				     XmNbottomAttachment, XmATTACH_FORM,
				     XmNsensitive, False,
				     NULL);

  XtAddCallback(w_button, XmNactivateCallback, ShellStopButtonCB, NULL);

  w_button = XtVaCreateManagedWidget("ShellInsSelButton", xmPushButtonWidgetClass, W_ShellDialog,
				     XmNrightAttachment, XmATTACH_WIDGET,
				     XmNrightWidget, w_button,
				     XmNbottomAttachment, XmATTACH_FORM,
				     NULL);

  XtAddCallback(w_button, XmNactivateCallback, ShellInsSelButtonCB, NULL);
  
  w_button = XtVaCreateManagedWidget("ShellExecuteButton", xmPushButtonWidgetClass, W_ShellDialog,
				     XmNrightAttachment, XmATTACH_WIDGET,
				     XmNrightWidget, w_button,
				     XmNbottomAttachment, XmATTACH_FORM,
				     XmNleftAttachment, XmATTACH_FORM,
				     NULL);

  XtVaSetValues(W_ShellDialog, XmNdefaultButton, w_button, NULL);

  XtAddCallback(w_button, XmNactivateCallback, ShellExecButtonCB, NULL);

  w_separator = XtVaCreateManagedWidget("ShellSeparator", xmSeparatorWidgetClass, W_ShellDialog,
					XmNleftAttachment, XmATTACH_FORM,
					XmNrightAttachment, XmATTACH_FORM,
					XmNbottomAttachment, XmATTACH_WIDGET,
					XmNbottomWidget, w_button,
					NULL);

  w_frame3 = XtVaCreateManagedWidget("ShellFrame3", xmFrameWidgetClass, W_ShellDialog,
				     XmNleftAttachment, XmATTACH_FORM,
				     XmNrightAttachment, XmATTACH_FORM,
				     XmNbottomAttachment, XmATTACH_WIDGET,
				     XmNbottomWidget, w_separator,
				     NULL);

   w_label = XtVaCreateManagedWidget("ShellLabel3", xmLabelWidgetClass, w_frame3,
				    XmNchildType, XmFRAME_TITLE_CHILD,
				    NULL);
 
  
  W_ShellCommand = XtVaCreateManagedWidget("ShellCommand", xmTextWidgetClass, w_frame3,
					   XmNchildType, XmFRAME_WORKAREA_CHILD,
					   NULL);

  w_pane = XtVaCreateManagedWidget("ShellPane", xmPanedWindowWidgetClass, W_ShellDialog,
				   XmNtopAttachment, XmATTACH_FORM,
				   XmNleftAttachment, XmATTACH_FORM,
				   XmNrightAttachment, XmATTACH_FORM,
				   XmNbottomAttachment, XmATTACH_WIDGET,
				   XmNbottomWidget, w_frame3,
				   XmNorientation, XmVERTICAL,
				   NULL);

  w_frame1 = XtVaCreateManagedWidget("ShellFrame1", xmFrameWidgetClass, w_pane,
				     /*				     XmNtopAttachment, XmATTACH_FORM,
				     XmNleftAttachment, XmATTACH_FORM,
				     XmNrightAttachment, XmATTACH_FORM,
				 				     XmNbottomAttachment, XmATTACH_POSITION,
				     XmNbottomPosition, 60,*/
				     NULL);

  w_label = XtVaCreateManagedWidget("ShellLabel1", xmLabelWidgetClass, w_frame1,
				    XmNchildType, XmFRAME_TITLE_CHILD,
				    NULL);

  w_scroll1 = XtVaCreateManagedWidget("ShellScroll1", xmScrolledWindowWidgetClass, w_frame1,
				      XmNchildType, XmFRAME_WORKAREA_CHILD,
				      XmNscrollingPolicy, XmAUTOMATIC,
				      NULL);

  W_ShellOutputText = XtVaCreateManagedWidget("ShellOutput", xmTextWidgetClass, w_scroll1,
					      XmNeditable, False,
					      XmNeditMode, XmMULTI_LINE_EDIT,
					      XmNautoShowCursorPosition, False,
					      XmNcursorPositionVisible, False,
					      NULL);
  
  w_frame2 = XtVaCreateManagedWidget("ShellFrame2", xmFrameWidgetClass, w_pane,
				     /*				     XmNtopAttachment, XmATTACH_WIDGET,
				     XmNtopWidget, w_frame1,
				     XmNleftAttachment, XmATTACH_FORM,
				     XmNrightAttachment, XmATTACH_FORM,
				     XmNbottomAttachment, XmATTACH_WIDGET,
				     XmNbottomWidget, w_frame3,*/
				     NULL);
  
  w_label = XtVaCreateManagedWidget("ShellLabel2", xmLabelWidgetClass, w_frame2,
				    XmNchildType, XmFRAME_TITLE_CHILD,
				    NULL);
  

  nbargs=0;
  XtSetArg(args[nbargs], XmNvisibleItemCount, 8);nbargs++;

  W_ShellHistoryList = XmCreateScrolledList(w_frame2, "ShellHistory", args, nbargs);
  XtManageChild(W_ShellHistoryList);
  XtVaSetValues(W_ShellHistoryList, 
		XmNselectionPolicy, XmSINGLE_SELECT,
		XmNvisibleItemCount, 8,
		NULL);

  XtAddCallback(W_ShellHistoryList, XmNdefaultActionCallback, ShellSelectHistoryCB, NULL);


  XtManageChild(W_ShellDialog);

}

void ShellRoutine(char *command, int autoexec)
{
  XmString xmcommand;
  int pos;

  /* create the dialog if necessary */

  if(W_ShellDialog==NULL)
    ShellInitDialog();
  
  XtManageChild(W_ShellDialog);

  /* if there is a command */

  if(command!=NULL) {
    
    if(ShellChildExist) {
      if(ConfirmDialog("Previous command still running", "Do you want to abort the previous command ?")) {
	if(ShellChildExist) {
	  kill(ShellChildPID, SIGKILL);
	  sleep(3);
	}
      } else return;
    } 

    XmTextSetString(W_ShellCommand, command);

    if(autoexec) {

      command=XmTextGetString(W_ShellCommand);

      if(command!=NULL && strcmp(command,"")!=0) {
	xmcommand=XmStringCreateSimple(command);

	pos = XmListItemPos(W_ShellHistoryList, xmcommand);

	if(pos!=0) XmListDeletePos(W_ShellHistoryList, pos);

	XmListAddItem(W_ShellHistoryList, xmcommand, 1);
	XmStringFree(xmcommand);
	XtVaSetValues(W_ShellHistoryList, XmNvisibleItemCount, 8, NULL);
	ShellExecCommand(command);
      }
      ShellExecCommand(command);
    }
  }
}

































































