
#define SORT_H

/****************************************************************

    Copyright (c) 1996-1997 Fred Pesch

    This program is free software; you can redistribute it 
    and/or modify it under the terms of the GNU General 
    Public License as published by the Free Software 
    Foundation (version 2 of the License).

    This program is distributed in the hope that it will 
    be useful, but WITHOUT ANY WARRANTY; without even the 
    implied warranty of MERCHANTABILITY or FITNESS FOR A 
    PARTICULAR PURPOSE.  See the GNU General Public License 
    for more details.

    For a copy of the GNU General Public License, write to the 
    Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
    MA 02139, USA.

****************************************************************/

#include <Xm/Xm.h>
#include <stdlib.h>

#include "interface.h"
#include "fileview.h"
#include "config.h"

#define NO_SORT 0
#define SORT_BY_NAME  1
#define SORT_BY_TYPE  2
#define SORT_BY_SIZE  3

extern TViewNode *ActualFileViewList;

int ActualSortType = NO_SORT;

int CompareByName(const TViewNode **file1, const TViewNode **file2)
{
  int result;
  
  if( (*file1)->filenode->file.type == DIRECTORY && (*file2)->filenode->file.type !=DIRECTORY) return(-1);
  if( (*file2)->filenode->file.type == DIRECTORY && (*file1)->filenode->file.type !=DIRECTORY) return(1);

  result = strcmp((*file1)->filenode->file.name, (*file2)->filenode->file.name);
  return(result);
}

int CompareByType(const TViewNode **file1, const TViewNode **file2)
{
  int result;

  if((*file1)->filenode->file.type<(*file2)->filenode->file.type)
    return(-1);
  if((*file1)->filenode->file.type>(*file2)->filenode->file.type)
    return(1);  

  if(GetFormatPriority((*file1)->filenode->file.name) < GetFormatPriority((*file2)->filenode->file.name)) return(-1);
  if(GetFormatPriority((*file1)->filenode->file.name) > GetFormatPriority((*file2)->filenode->file.name)) return(1);

  result = strcmp((*file1)->filenode->file.name, (*file2)->filenode->file.name);
  return(result);
}

int CompareBySize(const TViewNode **file1, const TViewNode **file2)
{
  int result;

  if( (*file1)->filenode->file.type == DIRECTORY && (*file2)->filenode->file.type !=DIRECTORY) return(-1);
  if( (*file2)->filenode->file.type == DIRECTORY && (*file1)->filenode->file.type !=DIRECTORY) return(1);

  if((*file1)->filenode->file.size<(*file2)->filenode->file.size)
    return(1);
  if((*file1)->filenode->file.size>(*file2)->filenode->file.size)
    return(-1);
  
  result = strcmp((*file1)->filenode->file.name, (*file2)->filenode->file.name);
  return(result);
}
 
void SortRoutine(void)
{
  TViewNode **filearray;
  TViewNode *actual;
  WidgetList w_temp;
  int i;
  int count,count2;
  int *tabindex;

  if(FileIconList.nb_icons_managed==0) return;

  filearray = (TViewNode **) XtMalloc(sizeof(TViewNode *)*FileIconList.nb_icons_managed);

  i=0;
  actual= ActualFileViewList;
  while(actual!=NULL) {
    if(actual->icon_index!=-1) {
      filearray[i]=(TViewNode *) XtMalloc(sizeof(TViewNode));
      filearray[i]->filenode=actual->filenode;
      filearray[i]->icon_index=actual->icon_index;
      i++;
    }
    actual=actual->next;
  }
  
  switch(ActualSortType) {

  case SORT_BY_NAME:
    qsort(filearray, i, sizeof(TViewNode *), (const void *) CompareByName);
    break;

  case SORT_BY_SIZE:
    qsort(filearray, i, sizeof(TViewNode *), (const void *)CompareBySize);
    break;

  case SORT_BY_TYPE:
    qsort(filearray, i, sizeof(TViewNode *), (const void *)CompareByType);
    break;

  default:
    ActualSortType = NO_SORT;
  }

  if(ActualSortType!=NO_SORT) {
    XtUnmanageChildren(FileIconList.w_icon, FileIconList.nb_icons_managed);

    w_temp = (WidgetList) XtMalloc(sizeof(Widget)*FileIconList.nb_icons_managed);
    
    for(i=0;i<FileIconList.nb_icons_managed;i++)
      w_temp[i]=FileIconList.w_icon[i];

    for(i=0;i<FileIconList.nb_icons_managed;i++) {
      FileIconList.w_icon[i]=w_temp[filearray[i]->icon_index];
    }

    tabindex = (int *) XtMalloc(sizeof(int)*FileIconList.nb_icons_managed);

    actual=ActualFileViewList;
    for(i=0;i<FileIconList.nb_icons_managed;i++) {   
      while(actual->icon_index==-1) actual=actual->next;
      tabindex[i]=actual->icon_index;
      actual=actual->next;
    }

    for(i=0;i<FileIconList.nb_icons_managed;i++) {      
      count=0;
      while(tabindex[count]!=filearray[i]->icon_index) count++;
      actual=ActualFileViewList;
      while(actual->icon_index==-1) actual=actual->next;
      for(count2=0;count2<count;count2++) actual=actual->next;    
      actual->icon_index=i;
    }

    XtFree((char *) tabindex);

    XtManageChildren(FileIconList.w_icon, FileIconList.nb_icons_managed);

    XmContainerReorder(W_ListFile, FileIconList.w_icon, FileIconList.nb_icons_managed);

    XmContainerRelayout(W_ListFile);

    XtFree((char *) w_temp);

  }

  actual=ActualFileViewList;
  while(actual->icon_index==-1) actual=actual->next;
  i=0;
  while(actual!=NULL) {
    XtFree((char *) filearray[i]);
    actual=actual->next;
    i++;
  }
  XtFree((char *) filearray);
}


